# -*- coding: utf-8 -*-
"""
Test repeat template and instance structure.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestRepeatTemplate(PyxformTestCase):
    """
    Ensuring the template and instance structure are added correctly.
    """

    def test_repeat_adding_template_and_instance(self):
        """
        Repeat should add template and instances
        """
        md = """
            | survey |              |         |                      |
            |        | type         | name    | label                |
            |        | text         | aa      | Text AA              |
            |        | begin repeat | section | Section              |
            |        | text         | a       | Text A               |
            |        | text         | b       | Text B               |
            |        | text         | c       | Text C               |
            |        | note         | d       | Note D               |
            |        | end repeat   |         |                      |
            |        |              |         |                      |
            |        | begin repeat | repeat_a| Section A            |
            |        | begin repeat | repeat_b| Section B            |
            |        | text         | e       | Text E               |
            |        | begin repeat | repeat_c| Section C            |
            |        | text         | f       | Text F               |
            |        | end repeat   |         |                      |
            |        | end repeat   |         |                      |
            |        | text         | g       | Text G               |
            |        | begin repeat | repeat_d| Section D            |
            |        | note         | h       | Note H               |
            |        | end repeat   |         |                      |
            |        | note         | i       | Note I               |
            |        | end repeat   |         |                      |
            """

        survey = self.md_to_pyxform_survey(md_raw=md)
        survey_xml = survey._to_pretty_xml()

        section_template = '<section jr:template="">'
        self.assertEqual(1, survey_xml.count(section_template))
        repeat_a_template = '<repeat_a jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_a_template))
        repeat_b_template = '<repeat_b jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_b_template))
        repeat_c_template = '<repeat_c jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_c_template))
        repeat_d_template = '<repeat_d jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_d_template))

        section_instance = "<section>"
        self.assertEqual(1, survey_xml.count(section_instance))
        repeat_a_instance = "<repeat_a>"
        self.assertEqual(1, survey_xml.count(repeat_a_instance))
        repeat_b_instance = "<repeat_b>"
        self.assertEqual(1, survey_xml.count(repeat_b_instance))
        repeat_c_instance = "<repeat_c>"
        self.assertEqual(1, survey_xml.count(repeat_c_instance))
        repeat_d_instance = "<repeat_d>"
        self.assertEqual(1, survey_xml.count(repeat_d_instance))

        self.assertPyxformXform(
            md=md,
            instance__contains=[
                '<section jr:template="">',
                '<repeat_a jr:template="">',
                '<repeat_b jr:template="">',
                '<repeat_c jr:template="">',
                '<repeat_d jr:template="">',
                "<section>",
                "<repeat_a>",
                "<repeat_b>",
                "<repeat_c>",
                "<repeat_d>",
            ],
        )

    def test_repeat_adding_template_and_instance_with_group(self):
        """
        Repeat should add template and instance even when they are inside grouping
        """
        md = """
            | survey |              |         |                      |
            |        | type         | name    | label                |
            |        | text         | aa      | Text AA              |
            |        | begin repeat | section | Section              |
            |        | text         | a       | Text A               |
            |        | text         | b       | Text B               |
            |        | text         | c       | Text C               |
            |        | note         | d       | Note D               |
            |        | end repeat   |         |                      |
            |        |              |         |                      |
            |        | begin group  | group_a | Group A              |
            |        | begin repeat | repeat_a| Section A            |
            |        | begin repeat | repeat_b| Section B            |
            |        | text         | e       | Text E               |
            |        | begin group  | group_b | Group B              |
            |        | text         | f       | Text F               |
            |        | text         | g       | Text G               |
            |        | note         | h       | Note H               |
            |        | end group    |         |                      |
            |        | note         | i       | Note I               |
            |        | end repeat   |         |                      |
            |        | end repeat   |         |                      |
            |        | end group    |         |                      |
            """

        survey = self.md_to_pyxform_survey(md_raw=md)
        survey_xml = survey._to_pretty_xml()

        section_template = '<section jr:template="">'
        self.assertEqual(1, survey_xml.count(section_template))
        repeat_a_template = '<repeat_a jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_a_template))
        repeat_b_template = '<repeat_b jr:template="">'
        self.assertEqual(1, survey_xml.count(repeat_b_template))

        section_instance = "<section>"
        self.assertEqual(1, survey_xml.count(section_instance))
        repeat_a_instance = "<repeat_a>"
        self.assertEqual(1, survey_xml.count(repeat_a_instance))
        repeat_b_instance = "<repeat_b>"
        self.assertEqual(1, survey_xml.count(repeat_b_instance))

        self.assertPyxformXform(
            md=md,
            instance__contains=[
                '<section jr:template="">',
                '<repeat_a jr:template="">',
                '<repeat_b jr:template="">',
                "<section>",
                "<repeat_a>",
                "<repeat_b>",
            ],
        )



# -*- coding: utf-8 -*-
"""
pyxform is a Python library designed to make authoring XForms for ODK
Collect easy.
"""

__version__ = "1.6.0"

from pyxform.builder import (
    SurveyElementBuilder,
    create_survey,
    create_survey_element_from_dict,
    create_survey_from_path,
    create_survey_from_xls,
)
from pyxform.instance import SurveyInstance
from pyxform.question import InputQuestion, MultipleChoiceQuestion, Question
from pyxform.question_type_dictionary import QUESTION_TYPE_DICT
from pyxform.section import Section
from pyxform.survey import Survey
from pyxform.xls2json import SurveyReader as ExcelSurveyReader

# This is what gets imported when someone imports pyxform
# flake8: noqa



# -*- coding: utf-8 -*-
"""
Testing the instance object for pyxform.
"""
from unittest import TestCase

from pyxform import Survey, SurveyInstance
from pyxform.builder import create_survey_element_from_dict
from pyxform.tests.utils import prep_class_config


class Json2XformExportingPrepTests(TestCase):
    @classmethod
    def setUpClass(cls):
        prep_class_config(cls=cls)

    def test_simple_survey_instantiation(self):
        surv = Survey(name="Simple")
        q = create_survey_element_from_dict(
            {"type": "text", "name": "survey_question", "label": "Question"}
        )
        surv.add_child(q)

        i = surv.instantiate()

        self.assertEquals(i.keys(), ["survey_question"])
        self.assertEquals(set(i.xpaths()), {"/Simple", "/Simple/survey_question"})

    def test_simple_survey_answering(self):
        surv = Survey(name="Water")
        q = create_survey_element_from_dict(
            {"type": "text", "name": "color", "label": "Color"}
        )
        q2 = create_survey_element_from_dict(
            {"type": "text", "name": "feeling", "label": "Feeling"}
        )

        surv.add_child(q)
        surv.add_child(q2)
        i = SurveyInstance(surv)

        i.answer(name="color", value="blue")
        self.assertEquals(i.answers()["color"], "blue")

        i.answer(name="feeling", value="liquidy")
        self.assertEquals(i.answers()["feeling"], "liquidy")

    def test_answers_can_be_imported_from_xml(self):
        surv = Survey(name="data")

        surv.add_child(
            create_survey_element_from_dict(
                {"type": "text", "name": "name", "label": "Name"}
            )
        )
        surv.add_child(
            create_survey_element_from_dict(
                {
                    "type": "integer",
                    "name": "users_per_month",
                    "label": "Users per month",
                }
            )
        )
        surv.add_child(
            create_survey_element_from_dict(
                {"type": "gps", "name": "geopoint", "label": "gps"}
            )
        )
        surv.add_child(
            create_survey_element_from_dict({"type": "imei", "name": "device_id"})
        )

        instance = surv.instantiate()
        import_xml = self.config.get(
            self.cls_name, "test_answers_can_be_imported_from_xml"
        )
        instance.import_from_xml(import_xml)

    def test_simple_registration_xml(self):
        reg_xform = Survey(name="Registration")
        name_question = create_survey_element_from_dict(
            {"type": "text", "name": "name", "label": "Name"}
        )
        reg_xform.add_child(name_question)

        reg_instance = reg_xform.instantiate()

        reg_instance.answer(name="name", value="bob")

        rx = reg_instance.to_xml()
        expected_xml = self.config.get(
            self.cls_name, "test_simple_registration_xml"
        ).format(reg_xform.id_string)
        self.assertEqual(rx, expected_xml)



# -*- coding: utf-8 -*-
"""
pyxform - Python library that converts XLSForms to XForms.
"""
from setuptools import find_packages, setup

REQUIRES = [
    "xlrd==1.2.0",
    "unicodecsv==0.14.1",
    "formencode==1.3.1",
    "unittest2==1.1.0",
    'functools32==3.2.3.post2 ; python_version < "3.2"',
]

setup(
    name="pyxform",
    version="1.6.0",
    author="github.com/xlsform",
    author_email="info@xlsform.org",
    packages=find_packages(),
    package_data={
        "pyxform.validators.odk_validate": ["bin/*.*"],
        "pyxform.tests": [
            "example_xls/*.*",
            "fixtures/strings.ini",
            "bug_example_xls/*.*",
            "test_output/*.*",
            "test_expected_output/*.*",
            "validators/.*",
            "validators/data/*.*",
            "validators/data/.*",
        ],
        "pyxform": ["iana_subtags.txt"],
    },
    url="http://pypi.python.org/pypi/pyxform/",
    description="A Python package to create XForms for ODK Collect.",
    long_description=open("README.rst", "rt").read(),
    install_requires=REQUIRES,
    entry_points={
        "console_scripts": [
            "xls2xform=pyxform.xls2xform:main_cli",
            "pyxform_validator_update=pyxform.validators.updater:main_cli",
        ]
    },
)



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestWarnings(PyxformTestCase):
    def test_warnings__count(self):
        """Should raise an expected number of warnings for a diverse form."""
        # Converted from warnings.xls file for tests/xlsform_spec_test/WarningsTest
        md = """
        | survey   |                        |                         |                           |         |                    |              |       |       |
        |          | type                   | name                    | label                     | hint    | appearance         | image        | audio | video |
        |          | text                   | some_text               |                           | a hint  |                    |              |       |       |
        |          | note                   | number_label            |                           | a note  |                    |              |       |       |
        |          | note                   | display_image_test      |                           |         |                    | img_test.jpg |       |       |
        |          | select_one yes_no      | autocomplete_test       | autocomplete_test         |         | autocomplete       |              |       |       |
        |          | select_one yes_no      | autocomplete_chars_test | autocomplete_chars_test   |         | autocomplete_chars |              |       |       |
        |          | integer                | a_integer               |                           | integer |                    |              |       |       |
        |          | decimal                | a_decimal               |                           | decimal |                    |              |       |       |
        |          | begin repeat           | repeat_test             |                           |         |                    |              |       |       |
        |          | begin group            | group_test              |                           |         |                    |              |       |       |
        |          | text                   | required_text           | required_text             |         |                    |              |       |       |
        |          | select_multiple yes_no | select_multiple_test    | select multiple test      |         | minimal            |              |       |       |
        |          | end group              | adsaf                   |                           |         |                    |              |       |       |
        |          | begin group            | labeled_select_group    | labeled select group test |         | field-list         |              |       |       |
        |          | end group              |                         |                           |         |                    |              |       |       |
        |          | begin group            | name                    |                           |         | table-list         |              |       |       |
        |          | select_one yes_no      | table_list_question     | table list question       | hint    |                    |              |       |       |
        |          | end group              |                         |                           |         |                    |              |       |       |
        |          | select_one a_b         | compact-test            |                           | hint    | compact            |              |       |       |
        |          | end repeat             |                         |                           |         |                    |              |       |       |
        |          | acknowledge            | acknowledge_test        |                           | hint    |                    |              |       |       |
        |          | date                   | date_test               |                           | hint    |                    |              |       |       |
        |          | time                   | time_test               |                           | hint    |                    |              |       |       |
        |          | datetime               | datetime_test           |                           | hint    |                    |              |       |       |
        |          | geopoint               | geopoint_test           |                           | hint    |                    |              |       |       |
        |          | barcode                | barcode_test            |                           | hint    |                    |              |       |       |
        |          | image                  | image_test              |                           | hint    |                    |              |       |       |
        |          | audio                  | audio_test              |                           | hint    |                    |              |       |       |
        |          | video                  | video_test              |                           | hint    |                    |              |       |       |
        |          | start                  | start                   |                           |         |                    |              |       |       |
        |          | end                    | end                     |                           |         |                    |              |       |       |
        |          | today                  | today                   |                           |         |                    |              |       |       |
        |          | deviceid               | deviceid                |                           |         |                    |              |       |       |
        |          | phonenumber            | phonenumber             |                           |         |                    |              |       |       |
        | choices  |           |         |       |       |
        |          | list_name | name    | label | image |
        |          | yes_no    | yes     | yes   |       |
        |          | yes_no    | no      | no    |       |
        |          | a_b       | a       |       | a.jpg |
        |          | a_b       | b       |       | b.jpg |
        |          | animals   | zebra   |       |       |
        |          | animals   | buffalo |       |       |
        | settings |            |           |            |                |                  |
        |          | form_title | form_id   | public_key | submission_url | default_language |
        |          | spec_test  | spec_test |            |                |                  |
        """  # noqa
        warnings = []
        self.assertPyxformXform(
            name="spec_test", md=md, warnings=warnings,
        )
        self.maxDiff = 2000
        expected = [
            "On the choices sheet there is a option with no label. [list_name : a_b]",
            "On the choices sheet there is a option with no label. [list_name : a_b]",
            "On the choices sheet there is a option with no label. [list_name : animals]",
            "On the choices sheet there is a option with no label. [list_name : animals]",
            "[row : 9] Repeat has no label: {'name': 'repeat_test', 'type': 'begin repeat'}",
            "[row : 10] Group has no label: {'name': 'group_test', 'type': 'begin group'}",
            "[row : 16] Group has no label: {'name': 'name', 'type': 'begin group'}",
            "[row : 27] Use the max-pixels parameter to speed up submission "
            + "sending and save storage space. Learn more: https://xlsform.org/#image",
        ]
        self.assertListEqual(expected, warnings)

    def test_warnings__unknown_control_group__with_name(self):
        """Should raise an error when an unknown control group is found."""
        self.assertPyxformXform(
            name="spec_test",
            md="""
            | survey   |               |         |
            |          | type          | name    |
            |          | begin dancing | dancing |
            """,
            errored=True,
            error__contains=["Unknown question type 'begin dancing'."],
        )

    def test_warnings__unknown_control_group__no_name(self):
        """Should raise an error when an unknown control group is found."""
        self.assertPyxformXform(
            name="spec_test",
            md="""
            | survey   |               |         |
            |          | type          | name    |
            |          | begin         | empty   |
            """,
            errored=True,
            error__contains=["Unknown question type 'begin'."],
        )



# -*- coding: utf-8 -*-
"""
Test XLSForm headers syntax.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class XlsFormHeadersTest(PyxformTestCase):
    def test_label_caps_alternatives(self):
        """
        re: https://github.com/SEL-Columbia/pyxform/issues/76
        Capitalization of 'label' column can lead to confusing errors.
        """
        s1 = self.md_to_pyxform_survey(
            """
            | survey |      |      |       |
            |        | type | name | label |
            |        | note | q    | Q     |
            """
        )
        s2 = self.md_to_pyxform_survey(
            """
            | survey |      |      |       |
            |        | type | name | Label | # <-- note: capital L
            |        | note | q    | Q     |
            """
        )
        self.assertEqual(s1.to_xml(), s2.to_xml())

    def test_calculate_alias(self):
        self.assertPyxformXform(
            name="calculatealias",
            md="""
            | survey |           |         |         |               |
            |        | type      | name    | label   | calculate     |
            |        | decimal   | amount  | Counter |               |
            |        | calculate | doubled | Doubled | ${amount} * 2 |
            """,
            errored=False,
            debug=False,
        )

    def test_form_id_variant(self):
        md = """
| survey       |                    |                |                |
|              | type               | name           | label          |
|              | text               | member_name    | name           |
| settings     |                    |              |                        |             |
|              | id_string                         | version                | form_id     |
|              | get_option_from_two_repeat_answer | vWvvk3GYzjXcJQyvTWELej | AUTO-v2-jef |
"""
        kwargs = {"name": "None", "title": "AUTO-v2-jef", "id_string": "AUTO-v2-jef"}

        survey = self.md_to_pyxform_survey(md, kwargs=kwargs, autoname=False)

        self.assertEqual(survey.id_string, "AUTO-v2-jef")
        self.assertEqual(survey.version, "vWvvk3GYzjXcJQyvTWELej")



# -*- coding: utf-8 -*-
"""
Test unicode rtl in XLSForms.
"""
from __future__ import unicode_literals

from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class UnicodeStrings(PyxformTestCase):
    def test_unicode_snowman(self):
        self.assertPyxformXform(
            md="""
            | survey |      |         |       |
            |        | type | name    | label |
            |        | text | snowman | ☃     |
            """,
            errored=False,
            xml__contains=["<label>☃</label>"],
        )

    def test_smart_quotes(self):
        self.assertPyxformXform(
            ss_structure={
                "survey": [
                    {
                        "type": "select_one xyz",
                        "name": "smart_single_quoted",
                        "label": """
                     ‘single-quoted’
                     """.strip(),
                    },
                    {
                        "type": "text",
                        "name": "smart_double_quoted",
                        "relevant": "selected(${smart_single_quoted}, ‘xxx’)",
                        "label": """
                     “double-quoted”
                     """.strip(),
                    },
                    {
                        "type": "integer",
                        "name": "my_default_is_123",
                        "label": "my default is 123",
                        "default": 123,
                    },
                ],
                "choices": [
                    {"list_name": "xyz", "name": "xxx", "label": "‘Xxx’"},
                    {"list_name": "xyz", "name": "yyy", "label": "“Yyy”"},
                ],
                "settings": [{"version": "q(‘-’)p"}],
            },
            errored=False,
            validate=False,
            name="quoth",
            xml__contains=[
                "'single-quoted",
                '"double-quoted"',
                "selected( /quoth/smart_single_quoted , 'xxx')",
                "<my_default_is_123>123</my_default_is_123>",
                "<label>'Xxx'</label>",
                '<label>"Yyy"</label>',
                """
                version="q('-')p"
                """.strip(),
            ],
        )



# -*- coding: utf-8 -*-
"""
Test whitespace around output variables in XForms.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class WhitespaceTest(PyxformTestCase):
    def test_over_trim(self):
        self.assertPyxformXform(
            name="issue96",
            md="""
            | survey  |                 |             |       |
            |         | type            | label       | name  |
            |         | text            | Ignored     | var   |
            |         | note            | ${var} text | label |
            """,
            xml__contains=['<label><output value=" /issue96/var "/> text </label>'],
        )

    def test_values_without_whitespaces_are_processed_successfully(self):
        md = """
            | survey  |                 |             |       |
            |         | type            | label       | name  |
            |         | text            | Ignored     | Var   |
            | settings       |                    |            |                                                   |
            |                | id_string          | public_key | submission_url                                    |
            |                | tutorial_encrypted | MIIB       | https://odk.ona.io/random_person/submission       |
          """

        survey = self.md_to_pyxform_survey(md_raw=md)
        expected = """<submission action="https://odk.ona.io/random_person/submission" base64RsaPublicKey="MIIB" method="post"/>"""
        xml = survey._to_pretty_xml()
        self.assertEqual(1, xml.count(expected))
        self.assertPyxformXform(md=md, xml__contains=expected, run_odk_validate=True)



# -*- coding: utf-8 -*-
"""
Test reapeat structure.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestRepeat(PyxformTestCase):
    """
    TestRepeat class.
    """

    def test_repeat_relative_reference(self):
        """
        Test relative reference in repeats.
        """
        self.assertPyxformXform(
            name="test_repeat",
            title="Relative Paths in repeats",
            md="""
                | survey |              |          |            |                      |
                |        | type         | name     | relevant   | label                |
                |        | text         | Z        |            | Fruit                |
                |        | begin repeat | section  |            | Section              |
                |        | text         | AA       |            | Anything really      |
                |        | text         | A        |            | A oat                |
                |        | text         | B        | ${A}='oat' | B w ${A}             |
                |        | note         | note1    |            | Noted ${AA} w ${A}   |
                |        | end repeat   |          |            |                      |
                |        |              |          |            |                      |
                |        | begin repeat | section2 |            | Section 2            |
                |        | text         | C        |            | C                    |
                |        | begin group  | sectiona |            | Section A            |
                |        | text         | D        |            | D oat                |
                |        | text         | E        | ${D}='oat' | E w ${Z}             |
                |        | note         | note2    |            | Noted ${C} w ${E}    |
                |        | end group    |          |            |                      |
                |        | note         | note3    |            | Noted ${C} w ${E}    |
                |        | end repeat   |          |            |                      |
                |        |              |          |            |                      |
                |        | begin repeat | section3 |            | Section 3            |
                |        | text         | FF       |            | F any text           |
                |        | text         | F        |            | F oat                |
                |        | begin group  | sectionb |            | Section B            |
                |        | text         | G        |            | G oat                |
                |        | text         | H        | ${G}='oat' | H w ${Z}             |
                |        | note         | note4    |            | Noted ${H} w ${Z}    |
                |        | end group    |          |            |                      |
                |        | begin repeat | sectionc |            | Section B            |
                |        | text         | I        |            | I                    |
                |        | text         | J        | ${I}='oat' | J w ${Z}             |
                |        | text         | K        | ${F}='oat' | K w ${Z}             |
                |        | text         | L        | ${G}='oat' | K w ${Z}             |
                |        | note         | note5    |            | Noted ${FF} w ${H}   |
                |        | note         | note6    |            | JKL #${J}#${K}#${L}  |
                |        | end repeat   |          |            |                      |
                |        | note         | note7    |            | Noted ${FF} w ${H}   |
                |        | begin group  | sectiond |            | Section D            |
                |        | text         | M        |            | M oat                |
                |        | text         | N        | ${G}='oat' | N w ${Z}             |
                |        | text         | O        | ${M}='oat' | O w ${Z}             |
                |        | note         | note8    |            | NO #${N} #${O}       |
                |        | end group    |          |            |                      |
                |        | note         | note9    |            | ${FF} ${H} ${N} ${N} |
                |        | end repeat   |          |            |                      |
                |        |              |          |            |                      |
                """,  # noqa pylint: disable=line-too-long
            instance__contains=[
                '<section jr:template="">',
                "<A/>",
                "<B/>",
                "</section>",
            ],
            model__contains=[
                """<bind nodeset="/test_repeat/section/A" """ """type="string"/>""",
                """<bind nodeset="/test_repeat/section/B" """
                """relevant=" ../A ='oat'" """
                """type="string"/>""",
                """<bind nodeset="/test_repeat/section2/sectiona/E" """
                """relevant=" ../D ='oat'" type="string"/>""",
                """<bind nodeset="/test_repeat/section3/sectionc/K" """
                """relevant=" ../../F ='oat'" type="string"/>""",
                """<bind nodeset="/test_repeat/section3/sectionc/L" """
                """relevant=" ../../sectionb/G ='oat'" """
                """type="string"/>""",
                """<bind nodeset="/test_repeat/section3/sectiond/N" """
                """relevant=" ../../sectionb/G ='oat'" """
                """type="string"/>""",
            ],
            xml__contains=[
                '<group ref="/test_repeat/section">',
                "<label>Section</label>",
                "</group>",
                """<label> B w <output value=" ../A "/> </label>""",
                """<label> E w <output value=" /test_repeat/Z "/> </label>""",
                """<label> Noted <output value=" ../FF "/> w """
                """<output value=" ../sectionb/H "/> </label></input>""",
            ],
        )

    def test_calculate_relative_path(self):
        """Test relative paths in calculate column."""
        self.assertPyxformXform(
            name="data",
            title="Paths in a calculate within a repeat are relative.",
            md="""
                | survey  |                      |       |        |                |
                |         | type                 | name  | label  | calculation    |
                |         | begin repeat         | rep   |        |                |
                |         | select_one crop_list | crop  | Select |                |
                |         | text                 | a     | Verify | name = ${crop} |
                |         | begin group          | group |        |                |
                |         | text                 | b     | Verify | name = ${crop} |
                |         | end group            |       |        |                |
                |         | end repeat           |       |        |                |
                |         |                      |       |        |                |
                | choices |                      |       |        |                |
                |         | list name            | name  | label  |                |
                |         | crop_list            | maize | Maize  |                |
                |         | crop_list            | beans | Beans  |                |
                |         | crop_list            | kale  | Kale   |                |
            """,  # noqa pylint: disable=line-too-long
            model__contains=[
                """<bind calculate="name =  ../crop " """
                """nodeset="/data/rep/a" type="string"/>""",
                """<bind calculate="name =  ../../crop " """
                """nodeset="/data/rep/group/b" type="string"/>""",
            ],
        )

    def test_choice_filter_relative_path(self):  # pylint: disable=invalid-name
        """Test relative paths in choice_filter column."""
        self.assertPyxformXform(
            name="data",
            title="Choice filter uses relative path",
            md="""
                | survey  |                      |       |        |                |
                |         | type                 | name  | label  | choice_filter  |
                |         | begin repeat         | rep   |        |                |
                |         | select_one crop_list | crop  | Select |                |
                |         | select_one crop_list | a     | Verify | name = ${crop} |
                |         | begin group          | group |        |                |
                |         | select_one crop_list | b     | Verify | name = ${crop} |
                |         | end group            |       |        |                |
                |         | end repeat           |       |        |                |
                |         |                      |       |        |                |
                | choices |                      |       |        |                |
                |         | list name            | name  | label  |                |
                |         | crop_list            | maize | Maize  |                |
                |         | crop_list            | beans | Beans  |                |
                |         | crop_list            | kale  | Kale   |                |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<itemset nodeset="instance('crop_list')/root/item[name =  current()/../crop ]">""",  # noqa pylint: disable=line-too-long
                """<itemset nodeset="instance('crop_list')/root/item[name =  current()/../../crop ]">""",  # noqa pylint: disable=line-too-long
            ],
        )

    def test_indexed_repeat_relative_path(self):
        """Test relative path not used with indexed-repeat()."""
        self.assertPyxformXform(
            name="data",
            title="Paths in a calculate within a repeat are relative.",
            md="""
                | survey  |                      |       |        |                                  |
                |         | type                 | name  | label  | calculation                      |
                |         | begin repeat         | rep   |        |                                  |
                |         | begin repeat         | rep2  |        |                                  |
                |         | select_one crop_list | crop  | Select |                                  |
                |         | text                 | a     | Verify |                                  |
                |         | begin group          | group |        |                                  |
                |         | text                 | b     | Verify |                                  |
                |         | end group            |       |        |                                  |
                |         | end repeat           |       |        |                                  |
                |         | calculate            | c1    |        | indexed-repeat(${a}, ${rep2}, 1) |
                |         | end repeat           |       |        |                                  |
                |         |                      |       |        |                                  |
                |         |                      |       |        |                                  |
                | choices |                      |       |        |                                  |
                |         | list name            | name  | label  |                                  |
                |         | crop_list            | maize | Maize  |                                  |
                |         | crop_list            | beans | Beans  |                                  |
                |         | crop_list            | kale  | Kale   |                                  |
            """,  # noqa pylint: disable=line-too-long
            model__contains=[
                """<bind calculate="indexed-repeat( /data/rep/rep2/a ,  /data/rep/rep2 , 1)" nodeset="/data/rep/c1" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_output_with_translation_relative_path(self):
        md = """
        | survey |              |             |                |              |               |                             |                           |                           |
        |        | type         | name        | label::English | calculation  | hint::English | constraint_message::English | required_message::English | noAppErrorString::English |
        |        | begin repeat | member      |                |              |               |                             |                           |                           |
        |        | calculate    | pos         |                | position(..) |               |                             |                           |                           |
        |        | text         | member_name | Name of ${pos} |              |               |                             |                           |                           |
        |        | text         | a           | A              |              | hint ${pos}   | constraint ${pos}           | required ${pos}           | app error ${pos}          |
        |        | end repeat   |             |                |              |               |                             |                           |                           |
        """

        self.assertPyxformXform(
            md=md,
            name="inside-repeat-relative-path",
            xml__contains=[
                '<translation lang="English">',
                '<value> Name of <output value=" ../pos "/> </value>',
                '<value> hint <output value=" ../pos "/> </value>',
                '<value> constraint <output value=" ../pos "/> </value>',
                '<value> required <output value=" ../pos "/> </value>',
                '<value> app error <output value=" ../pos "/>',
            ],
        )

    def test_output_with_guidance_hint_translation_relative_path(self):
        md = """
        | survey |              |             |                |                        |              |
        |        | type         | name        | label::English | guidance_hint::English | calculation  |
        |        | begin repeat | member      |                |                        |              |
        |        | calculate    | pos         |                |                        | position(..) |
        |        | text         | member_name | Name of ${pos} | More ${pos}            |              |
        |        | end repeat   |             |                |                        |              |
        """

        self.assertPyxformXform(
            md=md,
            name="inside-repeat-relative-path",
            xml__contains=[
                '<translation lang="English">',
                '<value> Name of <output value=" ../pos "/> </value>',
                '<value form="guidance"> More <output value=" ../pos "/> </value>',
            ],
        )

    def test_output_with_multiple_translations_relative_path(self):
        md = """
        | survey |              |                |                |                  |              |
        |        | type         | name           | label::English | label::Indonesia | calculation  |
        |        | begin repeat | member         |                |                  |              |
        |        | calculate    | pos            |                |                  | position(..) |
        |        | text         | member_name    | Name of ${pos} | Nama ${pos}      |              |
        |        | text         | member_address |                | Alamat           |              |
        |        | end repeat   |                |                |                  |              |
        """

        self.assertPyxformXform(
            md=md,
            name="inside-repeat-relative-path",
            xml__contains=[
                '<translation lang="English">',
                '<value> Name of <output value=" ../pos "/> </value>',
            ],
        )

    def test_hints_are_not_present_within_repeats(self):
        """Hints are not present within repeats"""
        md = """
            | survey |                   |                |                   |                      |
            |        | type              | name           | label             | hint                 |
            |        | begin repeat      | pets           | Pets              | Pet details          |
            |        | text              | pets_name      | Pet's name        | Pet's name hint      |
            |        | select_one pet    | pet_type       | Type of pet       | Type of pet hint     |
            |        | image             | pet_picture    | Picture of pet    | Take a nice photo    |
            |        | end repeat        |                |                   |                      |
            | choices|                   |                |                   |                      |
            |        | list name         | name           | label             |                      |
            |        | pet               | dog            | Dog               |                      |
            |        | pet               | cat            | Cat               |                      |
            |        | pet               | bird           | Bird              |                      |
            |        | pet               | fish           | Fish              |                      |
            """  # noqa

        expected = """
    <group ref="/pyxform_autotestname/pets">
      <label>Pets</label>
      <repeat nodeset="/pyxform_autotestname/pets">
        <input ref="/pyxform_autotestname/pets/pets_name">
          <label>Pet's name</label>
          <hint>Pet's name hint</hint>
        </input>
        <select1 ref="/pyxform_autotestname/pets/pet_type">
          <label>Type of pet</label>
          <hint>Type of pet hint</hint>
          <item>
            <label>Dog</label>
            <value>dog</value>
          </item>
          <item>
            <label>Cat</label>
            <value>cat</value>
          </item>
          <item>
            <label>Bird</label>
            <value>bird</value>
          </item>
          <item>
            <label>Fish</label>
            <value>fish</value>
          </item>
        </select1>
        <upload mediatype="image/*" ref="/pyxform_autotestname/pets/pet_picture">
          <label>Picture of pet</label>
          <hint>Take a nice photo</hint>
        </upload>
      </repeat>
    </group>
"""

        self.assertPyxformXform(md=md, xml__contains=[expected], run_odk_validate=True)

    def test_hints_are_present_within_groups(self):
        """Tests that hints are present within groups."""
        md = """
            | survey |                   |                        |                                                         |                              |
            |        | type              | name                   | label                                                   | hint                         |
            |        | begin group       | child_group            | Please enter birth information for each child born.     | Pet details                  |
            |        | text              | child_name             | Name of child?                                          | Should be a text             |
            |        | decimal           | birthweight            | Child birthweight (in kgs)?                             | Should be a decimal          |
            |        | end group         |                        |                                                         |                              |
            """  # noqa
        expected = """<group ref="/pyxform_autotestname/child_group">
      <label>Please enter birth information for each child born.</label>
      <input ref="/pyxform_autotestname/child_group/child_name">
        <label>Name of child?</label>
        <hint>Should be a text</hint>
      </input>
      <input ref="/pyxform_autotestname/child_group/birthweight">
        <label>Child birthweight (in kgs)?</label>
        <hint>Should be a decimal</hint>
      </input>
    </group>"""  # noqa

        self.assertPyxformXform(md=md, xml__contains=[expected], run_odk_validate=True)

    def test_choice_from_previous_repeat_answers(self):
        """Select one choices from previous repeat answers."""
        xlsform_md = """
        | survey  |                    |            |                |
        |         | type               | name       | label          |
        |         | begin repeat       | rep        | Repeat         |
        |         | text               | name       | Enter name     |
        |         | end repeat         |            |                |
        |         | select one ${name} | choice     | Choose name    |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                "<itemset nodeset=\"/data/rep[./name != '']\">",
                '<value ref="name"/>',
                '<label ref="name"/>',
            ],
        )

    def test_choice_from_previous_repeat_answers_not_name(self):
        """Select one choices from previous repeat answers."""
        xlsform_md = """
        | survey  |                      |            |                |
        |         | type                 | name       | label          |
        |         | begin repeat         | rep        | Repeat         |
        |         | text                 | answer     | Enter name     |
        |         | end repeat           |            |                |
        |         | select one ${answer} | choice     | Choose name    |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                "<itemset nodeset=\"/data/rep[./answer != '']\">",
                '<value ref="answer"/>',
                '<label ref="answer"/>',
            ],
        )

    def test_choice_from_previous_repeat_answers_with_choice_filter(self):
        """Select one choices from previous repeat answers with choice filter"""
        xlsform_md = """
        | survey  |                    |                |                |                           |
        |         | type               | name           | label          | choice_filter             |
        |         | begin repeat       | rep            | Repeat         |                           |
        |         | text               | name           | Enter name     |                           |
        |         | begin group        | demographics   | Demographics   |                           |
        |         | integer            | age            | Enter age      |                           |
        |         | end group          | demographics   |                |                           |
        |         | end repeat         |                |                |                           |
        |         | select one fruits  | fruit          | Choose a fruit |                           |
        |         | select one ${name} | choice         | Choose name    | starts-with(${name}, "b")  |
        |         | select one ${name} | choice_18_over | Choose name    | ${age} > 18               |
        | choices |                    |                |                |                           |
        |         | list name          | name           | label          |                           |
        |         | fruits             | banana         | Banana         |                           |
        |         | fruits             | mango          | Mango          |                           |
        """
        self.assertPyxformXform(
            name="data",
            id_string="some-id",
            md=xlsform_md,
            xml__contains=[
                '<itemset nodeset="/data/rep[starts-with( ./name , &quot;b&quot;)]">',
                '<itemset nodeset="/data/rep[ ./demographics/age  &gt; 18]">',
            ],
            run_odk_validate=False,
        )

    def test_choice_from_previous_repeat_answers_in_child_repeat(self):
        """
        Select one choice from previous repeat answers when within a child of a repeat
        """
        xlsform_md = """
        | survey  |                    |                           |                                                |                             |
        |         | type               | name                      | label                                          | choice_filter               |
        |         | begin repeat       | household                 | Household Repeat                               |                             |
        |         | begin repeat       | member                    | Household member repeat                        |                             |
        |         | text               | name                      | Enter name of a household member               |                             |
        |         | integer            | age                       | Enter age of the household member              |                             |
        |         | begin repeat       | adult                     | Select a representative                        |                             |
        |         | select one ${name} | adult_name                | Choose a name                                  | ${age} > 18                 |
        |         | end repeat         | adult                     |                                                |                             |
        |         | end repeat         | member                    |                                                |                             |
        |         | end repeat         | household                 |                                                |                             |
        """
        self.assertPyxformXform(
            name="data",
            id_string="some-id",
            md=xlsform_md,
            xml__contains=['<itemset nodeset="../../../member[ ./age  &gt; 18]">'],
        )

    def test_choice_from_previous_repeat_answers_in_nested_repeat(self):
        """Select one choices from previous repeat answers within a nested repeat"""
        xlsform_md = """
        | survey  |                    |                           |                                                |                             |
        |         | type               | name                      | label                                          | choice_filter               |
        |         | begin repeat       | household                 | Household Repeat                               |                             |
        |         | begin repeat       | person                    | Household member repeat                        |                             |
        |         | text               | name                      | Enter name of a household member               |                             |
        |         | integer            | age                       | Enter age of the household member              |                             |
        |         | end repeat         | person                    |                                                |                             |
        |         | begin repeat       | adult                     | Select a representative                        |                             |
        |         | select one ${name} | adult_name                | Choose a name                                  | ${age} > 18                 |
        |         | end repeat         | adult                     |                                                |                             |
        |         | end repeat         | household                 |                                                |                             |
        """
        self.assertPyxformXform(
            name="data",
            id_string="some-id",
            md=xlsform_md,
            xml__contains=['<itemset nodeset="../../person[ ./age  &gt; 18]">'],
        )

    def test_choice_from_previous_repeat_answers_in_nested_repeat_uses_current(self):
        """
        Select one choices from previous repeat answers within a nested repeat should use current if a sibling node of a select is used
        """
        xlsform_md = """
        | survey  |                    |                           |                                                |                             |
        |         | type               | name                      | label                                          | choice_filter               |
        |         | text               | enumerators_name          | Enter enumerators name                         |                             |
        |         | begin repeat       | household_rep             | Household Repeat                               |                             |
        |         | integer            | household_id              | Enter household ID                             |                             |
        |         | begin repeat       | household_mem_rep         | Household member repeat                        |                             |
        |         | text               | name                      | Enter name of a household member               |                             |
        |         | integer            | age                       | Enter age of the household member              |                             |
        |         | end repeat         | household_mem_rep         |                                                |                             |
        |         | begin repeat       | selected                  | Select a representative                        |                             |
        |         | integer            | target_min_age            | Minimum age requirement                        |                             |
        |         | select one ${name} | selected_name             | Choose a name                                  | ${age} > ${target_min_age}  |
        |         | end repeat         | selected                  |                                                |                             |
        |         | end repeat         | household_rep             |                                                |                             |
        """
        self.assertPyxformXform(
            name="data",
            id_string="some-id",
            md=xlsform_md,
            xml__contains=[
                '<itemset nodeset="../../household_mem_rep[ ./age  &gt;  current()/../target_min_age ]">',
            ],
        )

    def test_choice_from_previous_repeat_in_current_repeat_parents_out_to_repeat(self):
        """Test choice from previous repeat in current repeat produces the correct reference"""
        xlsform_md = """
        | survey       |                           |               |                        |                                                      |            |                       |              |
        |              | type                      | name          | label                  | choice_filter                                        | appearance | relevant              | calculation  |
        |              | begin_repeat              | pet           | Pet                    |                                                      | field_list |                       |              |
        |              | calculate                 | pos           |                        |                                                      |            |                       | position(..) |
        |              | select_one ${animal_type} | animal_select | Select the animal type | position() != current()/../pos and animal_type != '' |            |                       |              |
        |              | text                      | animal_type   | Animal type            |                                                      |            | ${animal_select} = '' |              |
        |              | end_repeat                | pet           |                        |                                                      |            |                       |              |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                "<itemset nodeset=\"../../pet[position() != current()/../pos and animal_type != '']\">"
            ],
        )

    def test_indexed_repeat_regular_calculation_relative_path_exception(self):
        """Test relative path exception (absolute path) in indexed-repeat() using regular calculation."""
        self.assertPyxformXform(
            name="data",
            title="regular calculation indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |           |                                  |                                              |
                |         | type           | name      | label                            | calculation                                  |
                |         | begin_repeat   | person    | Person                           |                                              |
                |         | integer        | pos       |                                  | position(..)                                 |
                |         | text           | name      | Enter name                       |                                              |
                |         | text           | prev_name | Name in previous repeat instance | indexed-repeat(${name}, ${person}, ${pos}-1) |
                |         | end repeat     |           |                                  |                                              |
            """,  # noqa pylint: disable=line-too-long
            model__contains=[
                """<bind calculate="indexed-repeat( /data/person/name ,  /data/person ,  ../pos -1)" nodeset="/data/person/prev_name" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_indexed_repeat_dynamic_default_relative_path_exception(self):
        """Test relative path exception (absolute path) in indexed-repeat() using dynamic default."""
        self.assertPyxformXform(
            name="data",
            title="dynamic default indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |           |                                  |                                                    |
                |         | type           | name      | label                            | default                                            |
                |         | begin_repeat   | person    | Person                           |                                                    |
                |         | text           | name      | Enter name                       |                                                    |
                |         | text           | prev_name | Name in previous repeat instance | indexed-repeat(${name}, ${person}, position(..)-1) |
                |         | end repeat     |           |                                  |                                                    |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<setvalue event="odk-instance-first-load odk-new-repeat" ref="/data/person/prev_name" value="indexed-repeat( /data/person/name ,  /data/person , position(..)-1)"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_indexed_repeat_nested_repeat_relative_path_exception(self):
        """Test relative path exception (absolute path) in indexed-repeat() using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |                |                                                        |                                                     |
                |         | type           | name           | label                                                  | default                                             |
                |         | begin_repeat   | family         | Family                                                 |                                                     |
                |         | integer        | members_number | How many members in this family?                       |                                                     |
                |         | begin_repeat   | person         | Person                                                 |                                                     |
                |         | text           | name           | Enter name                                             |                                                     |
                |         | text           | prev_name      | Non-sensible previous name in first family, 2nd person | indexed-repeat(${name}, ${family}, 1, ${person}, 2) |
                |         | end repeat     |                |                                                        |                                                     |
                |         | end repeat     |                |                                                        |                                                     |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<setvalue event="odk-instance-first-load odk-new-repeat" ref="/data/family/person/prev_name" value="indexed-repeat( /data/family/person/name ,  /data/family , 1,  /data/family/person , 2)"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_indexed_repeat_math_expression_nested_repeat_relative_path_exception(
        self,
    ):
        """Test relative path exception (absolute path) in indexed-repeat() with math expression using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |                |                                  |                                                        |
                |         | type           | name           | label                            | calculation                                            |
                |         | begin_repeat   | family         | Family                           |                                                        |
                |         | integer        | members_number | How many members in this family? |                                                        |
                |         | begin_repeat   | person         | Person                           |                                                        |
                |         | text           | name           | Enter name                       |                                                        |
                |         | integer        | age            | Enter age                        |                                                        |
                |         | text           | prev_name      | Expression label                 | 7 * indexed-repeat(${age}, ${family}, 1, ${person}, 2) |
                |         | end repeat     |                |                                  |                                                        |
                |         | end repeat     |                |                                  |                                                        |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<bind calculate="7 * indexed-repeat( /data/family/person/age ,  /data/family , 1,  /data/family/person , 2)" nodeset="/data/family/person/prev_name" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_multiple_indexed_repeat_in_expression_nested_repeat_relative_path_exception(
        self,
    ):
        """Test relative path exception (absolute path) in multiple indexed-repeat() inside an expression using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |                |                                  |                                                                                                                 |
                |         | type           | name           | label                            | required                                                                                                        |
                |         | begin_repeat   | family         | Family                           |                                                                                                                 |
                |         | integer        | members_number | How many members in this family? |                                                                                                                 |
                |         | begin_repeat   | person         | Person                           |                                                                                                                 |
                |         | text           | name           | Enter name                       |                                                                                                                 |
                |         | integer        | age            | Enter age                        |                                                                                                                 |
                |         | text           | prev_name      | Expression label                 | concat(indexed-repeat(${name}, ${family}, 1, ${person}, 2), indexed-repeat(${age}, ${family}, 1, ${person}, 2)) |
                |         | end repeat     |                |                                  |                                                                                                                 |
                |         | end repeat     |                |                                  |                                                                                                                 |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<bind nodeset="/data/family/person/prev_name" required="concat(indexed-repeat( /data/family/person/name ,  /data/family , 1,  /data/family/person , 2), indexed-repeat( /data/family/person/age ,  /data/family , 1,  /data/family/person , 2))" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_mixed_variables_and_indexed_repeat_in_expression_text_type_nested_repeat_relative_path_exception(
        self,
    ):
        """Test relative path exception (absolute path) in an expression contains variables and indexed-repeat() in a text type using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |                |                                  |                                                                                                                 |
                |         | type           | name           | label                            | calculation                                                                                                        |
                |         | begin_repeat   | family         | Family                           |                                                                                                                 |
                |         | integer        | members_number | How many members in this family? |                                                                                                                 |
                |         | begin_repeat   | person         | Person                           |                                                                                                                 |
                |         | text           | name           | Enter name                       |                                                                                                                 |
                |         | integer        | age            | Enter age                        |                                                                                                                 |
                |         | text           | prev_name      | Expression label                 | concat(${name}, indexed-repeat(${age}, ${family}, 1, ${person}, 2), ${age})                                     |
                |         | end repeat     |                |                                  |                                                                                                                 |
                |         | end repeat     |                |                                  |                                                                                                                 |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<bind calculate="concat( ../name , indexed-repeat( /data/family/person/age ,  /data/family , 1,  /data/family/person , 2),  ../age )" nodeset="/data/family/person/prev_name" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_mixed_variables_and_indexed_repeat_in_expression_integer_type_nested_repeat_relative_path_exception(
        self,
    ):
        """Test relative path exception (absolute path) in an expression contains variables and indexed-repeat() in an integer type using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |          |                        |                                                                             |
                |         | type           | name     | label                  | default                                                                     |
                |         | begin_group    | page     |                        |                                                                             |
                |         | begin_repeat   | bp_rg    |                        |                                                                             |
                |         | integer        | bp_row   | Repeating group entry  |                                                                             |
                |         | text           | bp_pos   | Position               |                                                                             |
                |         | integer        | bp_sys   | Systolic pressure      |                                                                             |
                |         | integer        | bp_dia   | Diastolic pressure     | if(${bp_row} = 1, '',  indexed-repeat(${bp_dia}, ${bp_rg}, ${bp_row} - 1))  |
                |         | end repeat     |          |                        |                                                                             |
                |         | end group      |          |                        |                                                                             |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<setvalue event="odk-instance-first-load odk-new-repeat" ref="/data/page/bp_rg/bp_dia" value="if( ../bp_row  = 1, '', indexed-repeat( /data/page/bp_rg/bp_dia ,  /data/page/bp_rg ,  ../bp_row  - 1))"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_indexed_repeat_math_expression_with_double_variable_in_nested_repeat_relative_path_exception(
        self,
    ):
        """Test relative path exception (absolute path) in indexed-repeat() with math expression and double variable using nested repeat."""
        self.assertPyxformXform(
            name="data",
            title="In nested repeat, indexed-repeat 1st, 2nd, 4th, and 6th argument is using absolute path",
            md="""
                | survey  |                |                |                                  |                                                             |
                |         | type           | name           | label                            | relevant                                                    |
                |         | begin_repeat   | family         | Family                           |                                                             |
                |         | integer        | members_number | How many members in this family? |                                                             |
                |         | begin_repeat   | person         | Person                           |                                                             |
                |         | text           | name           | Enter name                       |                                                             |
                |         | integer        | age            | Enter age                        |                                                             |
                |         | text           | prev_name      | Expression label                 | ${age} > indexed-repeat(${age}, ${family}, 1, ${person}, 2) |
                |         | end repeat     |                |                                  |                                                             |
                |         | end repeat     |                |                                  |                                                             |
            """,  # noqa pylint: disable=line-too-long
            xml__contains=[
                """<bind nodeset="/data/family/person/prev_name" relevant=" ../age  &gt; indexed-repeat( /data/family/person/age ,  /data/family , 1,  /data/family/person , 2)" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_using_select_with_reference_path_in_predicate_uses_current(self,):
        """
        Test relative path expansion using current if reference path is inside a predicate in a survey with select choice list
        """
        xlsform_md = """
        | survey |                 |              |                                                |               |                                                             |
        |        | type            | name         | label                                          | choice_filter | calculation                                                 |
        |        | begin repeat    | item-repeat  | Item                                           |               |                                                             |
        |        | calculate       | item-counter |                                                |               | position(..)                                                |
        |        | calculate       | item         |                                                |               | instance('item')/root/item[itemindex=${item-counter}]/label |
        |        | begin group     | item-info    | Item info                                      |               |                                                             |
        |        | note            | item-note    | All the following questions are about ${item}. |               |                                                             |
        |        | select one item | stock-item   | Do you stock this item?                        | true()        |                                                             |
        |        | end group       | item-info    |                                                |               |                                                             |
        |        | end repeat      |              |                                                |               |                                                             |
        | choices |           |                  |                   |           |
        |         | list_name | name             | label             | itemindex |
        |         | item      | gasoline-regular | Gasoline, Regular | 1         |
        |         | item      | gasoline-premium | Gasoline, Premium | 2         |
        |         | item      | gasoline-diesel  | Gasoline, Diesel  | 3         |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[itemindex= current()/../item-counter ]/label" nodeset="/data/item-repeat/item" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_using_select_uses_current_with_reference_path_in_predicate_and_instance_is_not_first_expression(
        self,
    ):
        """
        Test relative path expansion using current if reference path is inside a predicate and instance is not first expression in a survey with select choice list
        """
        xlsform_md = """
        | survey |                 |              |                                                |               |                                                                               |
        |        | type            | name         | label                                          | choice_filter | calculation                                                                   |
        |        | begin repeat    | item-repeat  | Item                                           |               |                                                                               |
        |        | calculate       | item-counter |                                                |               | position(..)                                                                  |
        |        | calculate       | item         |                                                |               | ${item-counter} + instance('item')/root/item[itemindex=${item-counter}]/label |
        |        | begin group     | item-info    | Item info                                      |               |                                                                               |
        |        | note            | item-note    | All the following questions are about ${item}. |               |                                                                               |
        |        | select one item | stock-item   | Do you stock this item?                        | true()        |                                                                               |
        |        | end group       | item-info    |                                                |               |                                                                               |
        |        | end repeat      |              |                                                |               |                                                                               |
        | choices |           |                  |                   |           |
        |         | list_name | name             | label             | itemindex |
        |         | item      | gasoline-regular | Gasoline, Regular | 1         |
        |         | item      | gasoline-premium | Gasoline, Premium | 2         |
        |         | item      | gasoline-diesel  | Gasoline, Diesel  | 3         |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate=" ../item-counter  + instance('item')/root/item[itemindex= current()/../item-counter ]/label" nodeset="/data/item-repeat/item" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_and_group_with_reference_path_in_predicate_uses_current(self,):
        """
        Test relative path expansion using current if reference path is inside a predicate in a survey with group
        """
        xlsform_md = """
        | survey |              |       |       |                                                 |
        |        | type         | name  | label | calculation                                     |
        |        | xml-external | item  |       |                                                 |
        |        | begin repeat | rep3  |       |                                                 |
        |        | calculate    | pos3  |       | position(..)                                    |
        |        | begin group  | grp3  |       |                                                 |
        |        | text         | txt3  | Enter |                                                 |
        |        | calculate    | item3 |       | instance('item')/root/item[index=${pos3}]/label |
        |        | end group    |       |       |                                                 |
        |        | end repeat   |       |       |                                                 |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index= current()/../../pos3 ]/label" nodeset="/data/rep3/grp3/item3" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_in_predicate_uses_current(self,):
        """
        Test relative path expansion using current if reference path is inside a predicate
        """
        xlsform_md = """
        | survey |              |       |       |                                                                 |
        |        | type         | name  | label | calculation                                                     |
        |        | xml-external | item  |       |                                                                 |
        |        | calculate    | pos1  |       | position(..)                                                    |
        |        | begin repeat | rep5  |       |                                                                 |
        |        | calculate    | pos5  |       | position(..)                                                    |
        |        | calculate    | item5 |       | instance('item')/root/item[index=${pos5} and ${pos1} = 1]/label |
        |        | end repeat   |       |       |                                                                 |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index= current()/../pos5  and  /data/pos1  = 1]/label" nodeset="/data/rep5/item5" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_with_spaces_in_predicate_uses_current(self,):
        """
        Test relative path expansion using current if reference path (with whitespaces before/after an operator of ${name}) is inside a predicate
        """
        xlsform_md = """
        | survey |              |       |       |                                                                   |
        |        | type         | name  | label | calculation                                                       |
        |        | xml-external | item  |       |                                                                   |
        |        | calculate    | pos1  |       | position(..)                                                      |
        |        | begin repeat | rep5  |       |                                                                   |
        |        | calculate    | pos5  |       | position(..)                                                      |
        |        | calculate    | item5 |       | instance('item')/root/item[index= ${pos5} and ${pos1} = 1]/label  |
        |        | calculate    | item6 |       | instance('item')/root/item[index =${pos5} and ${pos1} = 1]/label  |
        |        | calculate    | item7 |       | instance('item')/root/item[index = ${pos5} and ${pos1} = 1]/label |
        |        | end repeat   |       |       |                                                                   |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index=  current()/../pos5  and  /data/pos1  = 1]/label" nodeset="/data/rep5/item5" type="string"/>""",  # noqa pylint: disable=line-too-long
                """<bind calculate="instance('item')/root/item[index = current()/../pos5  and  /data/pos1  = 1]/label" nodeset="/data/rep5/item6" type="string"/>""",  # noqa pylint: disable=line-too-long
                """<bind calculate="instance('item')/root/item[index =  current()/../pos5  and  /data/pos1  = 1]/label" nodeset="/data/rep5/item7" type="string"/>""",  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_in_a_method_with_spaces_in_predicate_uses_current(
        self,
    ):
        """
        Test relative path expansion using current if reference path in a method (with whitespaces before/after an operator of ${name}) is inside apredicate
        """
        xlsform_md = """
        | survey |              |       |       |                                                                           |
        |        | type         | name  | label | calculation                                                               |
        |        | xml-external | item  |       |                                                                           |
        |        | calculate    | pos1  |       | position(..)                                                              |
        |        | begin repeat | rep5  |       |                                                                           |
        |        | calculate    | pos5  |       | position(..)                                                              |
        |        | calculate    | item5 |       | instance('item')/root/item[index=number(1+ ${pos5}) and ${pos1} = 1]/label |
        |        | end repeat   |       |       |                                                                           |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index=number(1+  current()/../pos5 ) and  /data/pos1  = 1]/label" nodeset="/data/rep5/item5" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_with_spaces_in_predicate_with_parenthesis_uses_current(
        self,
    ):
        """
        Test relative path expansion using current if reference path (with whitespaces before/after an operator of ${name}) is inside a predicate with parenthesis
        """
        xlsform_md = """
        | survey |              |       |       |                                                                   |
        |        | type         | name  | label | calculation                                                       |
        |        | xml-external | item  |       |                                                                   |
        |        | calculate    | pos1  |       | position(..)                                                      |
        |        | begin repeat | rep5  |       |                                                                   |
        |        | calculate    | pos5  |       | position(..)                                                      |
        |        | calculate    | item5 |       | instance('item')/root/item[index = ${pos5} and ${pos1} = 1]/label |
        |        | end repeat   |       |       |                                                                   |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index =  current()/../pos5  and  /data/pos1  = 1]/label" nodeset="/data/rep5/item5" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_relative_path_expansion_not_using_current_if_reference_path_is_predicate_but_not_in_a_repeat(
        self,
    ):
        """
        Test relative path expansion using xpath without current() if reference path is inside a predicate and not inside a repeat
        """
        xlsform_md = """
        | survey |              |       |       |                                                 |
        |        | type         | name  | label | calculation                                     |
        |        | xml-external | item  |       |                                                 |
        |        | calculate    | pos1  |       | position(..)                                    |
        |        | calculate    | item1 |       | instance('item')/root/item[index=${pos1}]/label |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index= /data/pos1 ]/label" nodeset="/data/item1" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_relative_path_expansion_not_using_current_if_reference_path_is_predicate_but_not_part_of_primary_instance(
        self,
    ):
        """
        Test relative path expansion using xpath without current() if reference path is inside a predicate but not part of the primary instance
        """
        xlsform_md = """
        | survey |              |       |       |                                   |
        |        | type         | name  | label | calculation                       |
        |        | xml-external | item  |       |                                   |
        |        | begin repeat | rep1  |       |                                   |
        |        | calculate    | pos2  |       | 1                                 |
        |        | calculate    | item2 |       | ${rep1}[number(${pos2})]/label    |
        |        | end repeat   |       |       |                                   |
                """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate=" /data/rep1 [number( ../pos2 )]/label" nodeset="/data/rep1/item2" type="string"/>"""  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_in_multiple_predicate_uses_current(self,):
        """
        Test relative path expansion using current if reference path is in multiple predicate
        """
        xlsform_md = """
        | survey |              |       |       |                                                                       |
        |        | type         | name  | label | calculation                                                           |
        |        | xml-external | item  |       |                                                                       |
        |        | begin repeat | rep5  |       |                                                                       |
        |        | calculate    | pos5  |       | position(..)                                                          |
        |        | calculate    | pos6  |       | position(..)                                                          |
        |        | calculate    | item5 |       | instance('item')/root/item[index = ${pos5}][position()=${pos6}]/label |
        |        | end repeat   |       |       |                                                                       |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index =  current()/../pos5 ][position()= current()/../pos6 ]/label" nodeset="/data/rep5/item5" type="string"/>""",  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_in_multiple_complex_predicate_uses_current(
        self,
    ):
        """
        Test relative path expansion using current if reference path is in multiple predicate, one of it is a complex one
        ${pos1} is not using current because it is not in repeat
        """
        xlsform_md = """
        | survey |              |       |       |                                                                                                       |
        |        | type         | name  | label | calculation                                                                                           |
        |        | xml-external | item  |       |                                                                                                       |
        |        | calculate    | pos1  |       | position(..)                                                                                          |
        |        | begin repeat | rep5  |       |                                                                                                       |
        |        | calculate    | pos5  |       | position(..)                                                                                          |
        |        | calculate    | pos6  |       | position(..)                                                                                          |
        |        | calculate    | item5 |       | instance('item')/root/item[index =${pos5} and selected('1 2 3 4', ${pos1})][position()=${pos6}]/label |
        |        | end repeat   |       |       |                                                                                                       |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="instance('item')/root/item[index = current()/../pos5  and selected('1 2 3 4',  /data/pos1 )][position()= current()/../pos6 ]/label" nodeset="/data/rep5/item5" type="string"/>""",  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_after_instance_in_predicate_uses_current(self,):
        """
        Test relative reference path expansion uses current if reference path is in predicate 
            even if the reference path is found after the instance() expression
        ${pos5} that is in 'index =${pos5}'' uses current because it is in a predicate
        ${pos5} that is in 'position()=${pos5}'' uses current because it is in a predicate (eventhough not in an instance)
        """
        xlsform_md = """
        | survey |              |       |       |                                                                                                |
        |        | type         | name  | label | calculation                                                                                    |
        |        | xml-external | item  |       |                                                                                                |
        |        | begin repeat | rep5  |       |                                                                                                |
        |        | calculate    | pos5  |       | position(..)                                                                                   |
        |        | calculate    | item5 |       | concat(instance('item')/root/item[index =${pos5}]/label, /data[position()=${pos5}]/text) |
        |        | end repeat   |       |       |                                                                                                |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="concat(instance('item')/root/item[index = current()/../pos5 ]/label, /data[position()= current()/../pos5 ]/text)" nodeset="/data/rep5/item5" type="string"/>""",  # noqa pylint: disable=line-too-long
            ],
        )

    def test_repeat_with_reference_path_after_instance_not_in_predicate_not_using_current(
        self,
    ):
        """
        Test relative reference path expansion not using current if reference path is not in predicate 
            and the reference path is found after the instance() expression
        ${pos5} that is in 'index =${pos5} uses current because it is in a predicate
        ${pos5} that is in '${pos5} + 1'' not using current because it is not in a predicate (regardless in an instance or not)
        """
        xlsform_md = """
        | survey |              |       |            |                                                                       |
        |        | type         | name  | label      | calculation                                                           |
        |        | xml-external | item  |            |                                                                       |
        |        | begin repeat | rep5  |            |                                                                       |
        |        | calculate    | pos5  |            | position(..)                                                          |
        |        | calculate    | item5 |            | concat(instance('item')/root/item[index =${pos5}]/label, ${pos5} + 1) |
        |        | end repeat   |       |            |                                                                       |
        """
        self.assertPyxformXform(
            name="data",
            md=xlsform_md,
            xml__contains=[
                """<bind calculate="concat(instance('item')/root/item[index = current()/../pos5 ]/label,  ../pos5  + 1)" nodeset="/data/rep5/item5" type="string"/>""",  # noqa pylint: disable=line-too-long
            ],
        )



# -*- coding: utf-8 -*-
"""
Testing simple cases for Xls2Json
"""
from unittest import TestCase

from pyxform.builder import create_survey_element_from_dict
from pyxform.tests import utils
from pyxform.xls2json import SurveyReader


class GroupTests(TestCase):
    def test_json(self):
        x = SurveyReader(utils.path_to_text_fixture("group.xls"), default_name="group")
        x_results = x.to_json_dict()
        expected_dict = {
            "name": "group",
            "title": "group",
            "id_string": "group",
            "sms_keyword": "group",
            "default_language": "default",
            "type": "survey",
            "children": [
                {
                    "name": "family_name",
                    "type": "text",
                    "label": {"English": "What's your family name?"},
                },
                {
                    "name": "father",
                    "type": "group",
                    "label": {"English": "Father"},
                    "children": [
                        {
                            "name": "phone_number",
                            "type": "phone number",
                            "label": {"English": "What's your father's phone number?"},
                        },
                        {
                            "name": "age",
                            "type": "integer",
                            "label": {"English": "How old is your father?"},
                        },
                    ],
                },
                {
                    "children": [
                        {
                            "bind": {"jr:preload": "uid", "readonly": "true()"},
                            "name": "instanceID",
                            "type": "calculate",
                        }
                    ],
                    "control": {"bodyless": True},
                    "name": "meta",
                    "type": "group",
                },
            ],
        }
        self.maxDiff = None
        self.assertEqual(x_results, expected_dict)

    def test_equality_of_to_dict(self):
        x = SurveyReader(utils.path_to_text_fixture("group.xls"), default_name="group")
        x_results = x.to_json_dict()

        survey = create_survey_element_from_dict(x_results)
        survey_dict = survey.to_json_dict()
        # using the builder sets the title attribute to equal name
        # this won't happen through reading the excel file as done by
        # SurveyReader.
        # Now it happens.
        # del survey_dict[u'title']
        self.maxDiff = None
        self.assertEqual(x_results, survey_dict)



# -*- coding: utf-8 -*-
"""
Test language warnings.
"""
import os
import tempfile

from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class LanguageWarningTest(PyxformTestCase):
    """
    Test language warnings.
    """

    def test_label_with_valid_subtag_should_not_warn(self):
        survey = self.md_to_pyxform_survey(
            """
            | survey |      |         |                     |
            |        | type | name    | label::English (en) |
            |        | note | my_note | My note             |
            """
        )

        warnings = []
        tmp = tempfile.NamedTemporaryFile(suffix=".xml", delete=False)
        tmp.close()
        survey.print_xform_to_file(tmp.name, warnings=warnings)

        self.assertTrue(len(warnings) == 0)
        os.unlink(tmp.name)

    def test_label_with_no_subtag_should_warn(self):
        survey = self.md_to_pyxform_survey(
            """
            | survey |      |         |                     |
            |        | type | name    | label::English      |
            |        | note | my_note | My note             |
            """
        )

        warnings = []
        tmp = tempfile.NamedTemporaryFile(suffix=".xml", delete=False)
        tmp.close()
        survey.print_xform_to_file(tmp.name, warnings=warnings)

        self.assertTrue(len(warnings) == 1)
        self.assertTrue(
            "do not contain valid machine-readable codes: English. Learn more"
            in warnings[0]
        )
        os.unlink(tmp.name)

    def test_label_with_unknown_subtag_should_warn(self):
        survey = self.md_to_pyxform_survey(
            """
            | survey |      |         |                       |
            |        | type | name    | label::English (schm) |
            |        | note | my_note | My note               |
            """
        )

        warnings = []
        tmp = tempfile.NamedTemporaryFile(suffix=".xml", delete=False)
        tmp.close()
        survey.print_xform_to_file(tmp.name, warnings=warnings)

        self.assertTrue(len(warnings) == 1)
        self.assertTrue(
            "do not contain valid machine-readable codes: English (schm). Learn more"
            in warnings[0]
        )
        os.unlink(tmp.name)

    def test_default_language_only_should_not_warn(self):
        survey = self.md_to_pyxform_survey(
            """
            | survey |                 |         |        |               |
            |        | type            | name    | label  | choice_filter |
            |        | select_one opts | opt     | My opt | fake = 1      |
            | choices|                 |         |        |               |
            |        | list_name       | name    | label  | fake          |
            |        | opts            | opt1    | Opt1   | 1             |
            |        | opts            | opt2    | Opt2   | 1             |
            """
        )

        warnings = []
        tmp = tempfile.NamedTemporaryFile(suffix=".xml", delete=False)
        tmp.close()
        survey.print_xform_to_file(tmp.name, warnings=warnings)

        self.assertTrue(len(warnings) == 0)
        os.unlink(tmp.name)



# -*- coding: utf-8 -*-
"""
Test settins auto settings.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class SettingsAutoSendDelete(PyxformTestCase):
    def test_settings_auto_send_true(self):

        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |              |           |           |
            |          | type         | name      | label     |
            |          | text         | name      | Name      |
            | settings |              |           |           |
            |          | auto_send    |           |           |
            |          | true         |           |           |
            """,
            debug=False,
            xml__contains=['<submission orx:auto-send="true"/>'],
        )

    def test_settings_auto_delete_true(self):

        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |              |           |           |
            |          | type         | name      | label     |
            |          | text         | name      | Name      |
            | settings |              |           |           |
            |          | auto_delete  |           |           |
            |          | true         |           |           |
            """,
            debug=False,
            xml__contains=['<submission orx:auto-delete="true"/>'],
        )

    def test_settings_auto_send_delete_false(self):

        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |              |           |           |
            |          | type         | name      | label     |
            |          | text         | name      | Name      |
            | settings |              |           |           |
            |          | auto_delete  | auto_send |           |
            |          | false        | false     |           |
            """,
            debug=False,
            xml__contains=[
                '<submission orx:auto-delete="false" orx:auto-send="false"/>'
            ],
        )

    def test_settings_without_submission_url_does_not_generate_method_attribute(self):

        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |              |           |           |
            |          | type         | name      | label     |
            |          | text         | name      | Name      |
            | settings |                                                                    |           |           |
            |          | public_key                                                         | auto_send |           |
            |          | MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwOHPJWD9zc8JPBZj/UtC   | false     |           |
            |          | dHiY7I4HWt61UG1XRaGvvwUkC/y8P5Kk6dRnf3yMTBHQoisT2vU2ODWVaU5elndk   |           |           |
            |          | hiKiWdhufp1d86FWGYz/i+VOmdoV+0zoyPzk+vTEG8bpiY7/UcDYY0CsrRmaMei1   |           |           |
            |          | 15xZwQpSMpayqMjemvwGDyhy2B3Yize4yaxyLFG53wMrHEczzsYz8FuRfuKUleE/   |           |           |
            |          | 6jFc3uXZET4LJ7S76n1XU+bE+mhhoZ+tVERgaVH38l0SZljBITwHeqQ9WQckkmDf   |           |           |
            |          | bRHBG7TQm+Afnx0s5E2bGIT5jB5cj9YaX6BqZSeodpafQjpXEJg6uufxF1Ni3Btv   |           |           |
            |          |  4wIDAQAB                                                          |           |           |
            """,
            debug=False,
            xml__contains=[
                '<submission base64RsaPublicKey="MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAwOHPJWD9zc8JPBZj/UtC" orx:auto-send="false"/>'
            ],
        )

    def test_settings_with_submission_url_generates_method_attribute(self):

        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |              |           |           |
            |          | type         | name      | label     |
            |          | text         | name      | Name      |
            | settings |                                                       |           |           |
            |          | submission_url                                        | auto_send |           |
            |          | https://odk.ona.io/random_person/submission           | false     |           |
            """,
            debug=False,
            xml__contains=[
                '<submission action="https://odk.ona.io/random_person/submission" method="post" orx:auto-send="false"/>'
            ],
        )



# -*- coding: utf-8 -*-
"""
XForm survey question type mapping dictionary module.
"""
from pyxform.xls2json import QuestionTypesReader, print_pyobj_to_json


def generate_new_dict():
    """
    This is just here incase there is ever any need to generate the question
    type dictionary from all.xls again.
    It shouldn't be called as part of any application.
    """
    path_to_question_types = "/pyxform/question_types/all.xls"
    json_dict = QuestionTypesReader(path_to_question_types).to_json_dict()
    print_pyobj_to_json(json_dict, "new_question_type_dict.json")


QUESTION_TYPE_DICT = {
    "q picture": {
        "control": {"tag": "upload", "mediatype": "image/*"},
        "bind": {"type": "binary"},
    },
    "photo": {
        "control": {"tag": "upload", "mediatype": "image/*"},
        "bind": {"type": "binary"},
    },
    "add date time prompt": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "add audio prompt": {
        "control": {"tag": "upload", "mediatype": "audio/*"},
        "bind": {"type": "binary"},
    },
    "q date time": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "phonenumber": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "phonenumber",
        }
    },
    "get start time": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "start",
        }
    },
    "add select multiple prompt using": {
        "control": {"tag": "select"},
        "bind": {"type": "string"},
    },
    "add note prompt": {
        "control": {"tag": "input"},
        "bind": {"readonly": "true()", "type": "string"},
    },
    "calculate": {"bind": {"type": "string"}},
    "acknowledge": {"control": {"tag": "trigger"}, "bind": {"type": "string"}},
    "location": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "text": {"control": {"tag": "input"}, "bind": {"type": "string"}},
    "select all that apply from": {
        "control": {"tag": "select"},
        "bind": {"type": "string"},
    },
    "simserial": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "simserial",
        }
    },
    "string": {"control": {"tag": "input"}, "bind": {"type": "string"}},
    "q string": {"control": {"tag": "input"}, "bind": {"type": "string"}},
    "imei": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "deviceid",
        }
    },
    "integer": {"control": {"tag": "input"}, "bind": {"type": "int"}},
    "datetime": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "q note": {
        "control": {"tag": "input"},
        "bind": {"readonly": "true()", "type": "string"},
    },
    "subscriber id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "subscriberid",
        }
    },
    "decimal": {"control": {"tag": "input"}, "bind": {"type": "decimal"}},
    "dateTime": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "q audio": {
        "control": {"tag": "upload", "mediatype": "audio/*"},
        "bind": {"type": "binary"},
    },
    "q geopoint": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "q geoshape": {"control": {"tag": "input"}, "bind": {"type": "geoshape"}},
    "q geotrace": {"control": {"tag": "input"}, "bind": {"type": "geotrace"}},
    "q image": {
        "control": {"tag": "upload", "mediatype": "image/*"},
        "bind": {"type": "binary"},
    },
    "get today": {
        "bind": {"jr:preload": "date", "type": "date", "jr:preloadParams": "today"}
    },
    "video": {
        "control": {"tag": "upload", "mediatype": "video/*"},
        "bind": {"type": "binary"},
    },
    "q acknowledge": {"control": {"tag": "trigger"}, "bind": {"type": "string"}},
    "add video prompt": {
        "control": {"tag": "upload", "mediatype": "video/*"},
        "bind": {"type": "binary"},
    },
    "number of days in last month": {
        "control": {"tag": "input"},
        "bind": {"type": "int", "constraint": "0 <= . and . <= 31"},
        "hint": "Enter a number 0-31.",
    },
    "get sim id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "simserial",
        }
    },
    "q location": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "select one": {"control": {"tag": "select1"}, "bind": {"type": "string"}},
    "select one external": {"control": {"tag": "input"}, "bind": {"type": "string"}},
    "add image prompt": {
        "control": {"tag": "upload", "mediatype": "image/*"},
        "bind": {"type": "binary"},
    },
    "select all that apply": {"control": {"tag": "select"}, "bind": {"type": "string"}},
    "get end time": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "end",
        }
    },
    "barcode": {"control": {"tag": "input"}, "bind": {"type": "barcode"}},
    "q video": {
        "control": {"tag": "upload", "mediatype": "video/*"},
        "bind": {"type": "binary"},
    },
    "geopoint": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "geoshape": {"control": {"tag": "input"}, "bind": {"type": "geoshape"}},
    "geotrace": {"control": {"tag": "input"}, "bind": {"type": "geotrace"}},
    "select multiple from": {"control": {"tag": "select"}, "bind": {"type": "string"}},
    "end time": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "end",
        }
    },
    "device id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "deviceid",
        }
    },
    "subscriberid": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "subscriberid",
        }
    },
    "q barcode": {"control": {"tag": "input"}, "bind": {"type": "barcode"}},
    "q select": {"control": {"tag": "select"}, "bind": {"type": "string"}},
    "select one using": {"control": {"tag": "select1"}, "bind": {"type": "string"}},
    "rank": {"control": {"tag": "odk:rank"}, "bind": {"type": "odk:rank"}},
    "image": {
        "control": {"tag": "upload", "mediatype": "image/*"},
        "bind": {"type": "binary"},
    },
    "q int": {"control": {"tag": "input"}, "bind": {"type": "int"}},
    "add text prompt": {"control": {"tag": "input"}, "bind": {"type": "string"}},
    "add date prompt": {"control": {"tag": "input"}, "bind": {"type": "date"}},
    "q calculate": {"bind": {"type": "string"}},
    "start": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "start",
        }
    },
    "trigger": {"control": {"tag": "trigger"}},
    "add acknowledge prompt": {
        "control": {"tag": "trigger"},
        "bind": {"type": "string"},
    },
    "percentage": {
        "control": {"tag": "input"},
        "bind": {"type": "int", "constraint": "0 <= . and . <= 100"},
    },
    "get phone number": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "phonenumber",
        }
    },
    "today": {
        "bind": {"jr:preload": "date", "type": "date", "jr:preloadParams": "today"}
    },
    "gps": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "q date": {"control": {"tag": "input"}, "bind": {"type": "date"}},
    "sim id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "simserial",
        }
    },
    "add decimal prompt": {"control": {"tag": "input"}, "bind": {"type": "decimal"}},
    "number of days in last six months": {
        "control": {"tag": "input"},
        "bind": {"type": "int", "constraint": "0 <= . and . <= 183"},
        "hint": "Enter a number 0-183.",
    },
    "deviceid": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "deviceid",
        }
    },
    "int": {"control": {"tag": "input"}, "bind": {"type": "int"}},
    "add barcode prompt": {"control": {"tag": "input"}, "bind": {"type": "barcode"}},
    "select multiple using": {"control": {"tag": "select"}, "bind": {"type": "string"}},
    "q decimal": {"control": {"tag": "input"}, "bind": {"type": "decimal"}},
    "end": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "end",
        }
    },
    "add calculate prompt": {"bind": {"type": "string"}},
    "add dateTime prompt": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "note": {
        "control": {"tag": "input"},
        "bind": {"readonly": "true()", "type": "string"},
    },
    "add location prompt": {"control": {"tag": "input"}, "bind": {"type": "geopoint"}},
    "get subscriber id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "subscriberid",
        }
    },
    "phone number": {
        "control": {"tag": "input"},
        "bind": {"type": "string", "constraint": "regex(., '^\\d*$')"},
        "hint": "Enter numbers only.",
    },
    "get device id": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "deviceid",
        }
    },
    "add integer prompt": {"control": {"tag": "input"}, "bind": {"type": "int"}},
    "q dateTime": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "date": {"control": {"tag": "input"}, "bind": {"type": "date"}},
    "q select1": {"control": {"tag": "select1"}, "bind": {"type": "string"}},
    "start time": {
        "bind": {
            "jr:preload": "timestamp",
            "type": "dateTime",
            "jr:preloadParams": "start",
        }
    },
    "number of days in last year": {
        "control": {"tag": "input"},
        "bind": {"type": "int", "constraint": "0 <= . and . <= 365"},
        "hint": "Enter a number 0-365.",
    },
    "date time": {"control": {"tag": "input"}, "bind": {"type": "dateTime"}},
    "time": {"control": {"tag": "input"}, "bind": {"type": "time"}},
    "audio": {
        "control": {"tag": "upload", "mediatype": "audio/*"},
        "bind": {"type": "binary"},
    },
    "add select one prompt using": {
        "control": {"tag": "select1"},
        "bind": {"type": "string"},
    },
    "hidden": {"bind": {"type": "string"}},
    "uri:subscriberid": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:subscriberid",
        }
    },
    "uri:phonenumber": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:phonenumber",
        }
    },
    "uri:simserial": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:simserial",
        }
    },
    "uri:deviceid": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:deviceid",
        }
    },
    "username": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "username",
        }
    },
    "uri:username": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:username",
        }
    },
    "email": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "email",
        }
    },
    "uri:email": {
        "bind": {
            "jr:preload": "property",
            "type": "string",
            "jr:preloadParams": "uri:email",
        }
    },
    "osm": {
        "control": {"tag": "upload", "mediatype": "osm/*"},
        "bind": {"type": "binary"},
    },
    "file": {
        "control": {"tag": "upload", "mediatype": "application/*"},
        "bind": {"type": "binary"},
    },
    "add file prompt": {
        "control": {"tag": "upload", "mediatype": "application/*"},
        "bind": {"type": "binary"},
    },
    "range": {"control": {"tag": "range"}, "bind": {"type": "int"}},
    "audit": {"bind": {"type": "binary"}},
    "xml-external": {
        # Only effect is to add an external instance.
    },
    "csv-external": {
        # Only effect is to add an external instance.
    },
    "start-geopoint": {
        "control": {"tag": "action"},
        "bind": {"type": "geopoint"},
        "action": {"name": "odk:setgeopoint", "event": "odk-instance-first-load"},
    },
    "background-audio": {
        "control": {"tag": "action"},
        "bind": {"type": "binary"},
        "action": {"name": "odk:recordaudio", "event": "odk-instance-load"},
    },
}



# -*- coding: utf-8 -*-
"""
Test XForm2JSON functionality
"""
import json

from pyxform.builder import create_survey_element_from_dict
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestXForm2JSON(PyxformTestCase):
    """
    Test xform2json module
    """

    def test_convert_toJSON_multi_language(self):
        """
        Test that it's possible to convert XLSForms with multiple languages
        to JSON and back into XML without losing any of the required information
        """
        md = """
        | survey  |
        |         | type                   | name  | label:Eng  | label:Fr |
        |         | text                   | name  | Name       | Prénom   |
        |         | select_multiple fruits | fruit | Fruit      | Fruit    |
        |         |                        |       |            |          |
        | choices | list name              | name  | label:Eng  | label:Fr |
        |         | fruits                 | 1     | Mango      | Mangue   |
        |         | fruits                 | 2     | Orange     | Orange   |
        |         | fruits                 | 3     | Apple      | Pomme    |
        """

        survey = self.md_to_pyxform_survey(
            md_raw=md,
            kwargs={"id_string": "id", "name": "multi-language", "title": "some-title"},
            autoname=False,
        )
        expected_xml = survey.to_xml()
        generated_json = survey.to_json()

        survey = create_survey_element_from_dict(json.loads(generated_json))

        self.assertEqual(expected_xml, survey.to_xml())



# -*- coding: utf-8 -*-
"""
Test sms syntax.
"""
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class SMSTest(PyxformTestCase):
    def test_prefix_only(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |           |          |       |           |
            |          | type      |   name   | label | hint      |
            |          | string    |   name   | Name  | your name |
            | settings |           |          |       |           |
            |          | prefix    |          |       |           |
            |          | sms_test  |          |       |           |
            """,
            xml__contains=['odk:prefix="sms_test"'],
        )

    def test_delimiter_only(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |           |          |       |           |
            |          | type      |   name   | label | hint      |
            |          | string    |   name   | Name  | your name |
            | settings |           |          |       |           |
            |          | delimiter |          |       |           |
            |          | ~         |          |       |           |
            """,
            xml__contains=['odk:delimiter="~"'],
        )

    def test_prefix_and_delimiter(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |           |          |       |           |
            |          | type      |   name   | label | hint      |
            |          | string    |   name   | Name  | your name |
            | settings |           |          |       |           |
            |          | delimiter | prefix   |       |           |
            |          | *         | sms_test2|       |           |
            """,
            xml__contains=['odk:delimiter="*"', 'odk:prefix="sms_test2"'],
        )

    def test_sms_tag(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey   |           |          |             |       |           |         |
            |          | type      |   name   | compact_tag | label | hint      | default |
            |          | string    |   name   | n           | Name  | your name |         |
            |          | int       |   age    | +a          | Age   | your age  | 7       |
            |          | string    | fruit    |             | Fruit | fav fruit |         |
            """,
            xml__contains=[
                '<name odk:tag="n"/>',
                '<age odk:tag="+a">7</age>',
                "<fruit/>",
            ],
        )



# -*- coding: utf-8 -*-
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class AudioQualityTest(PyxformTestCase):
    def test_voice_only(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                |
            |        | type   | name     | label | parameters     |
            |        | audio  | audio    | Audio | quality=voice-only |
            """,
            xml__contains=[
                'xmlns:orx="http://openrosa.org/xforms"',
                '<bind nodeset="/data/audio" type="binary" odk:quality="voice-only"/>',
            ],
        )

    def test_low(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                |
            |        | type   | name     | label | parameters     |
            |        | audio  | audio    | Audio | quality=low |
            """,
            xml__contains=[
                'xmlns:orx="http://openrosa.org/xforms"',
                '<bind nodeset="/data/audio" type="binary" odk:quality="low"/>',
            ],
        )

    def test_normal(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                |
            |        | type   | name     | label | parameters     |
            |        | audio  | audio    | Audio | quality=normal |
            """,
            xml__contains=[
                'xmlns:orx="http://openrosa.org/xforms"',
                '<bind nodeset="/data/audio" type="binary" odk:quality="normal"/>',
            ],
        )

    def test_external(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                |
            |        | type   | name     | label | parameters     |
            |        | audio  | audio    | Audio | quality=external |
            """,
            xml__contains=[
                'xmlns:orx="http://openrosa.org/xforms"',
                '<bind nodeset="/data/audio" type="binary" odk:quality="external"/>',
            ],
        )

    def test_foo_fails(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                |
            |        | type   | name     | label | parameters     |
            |        | audio  | audio    | Audio | quality=foo |
            """,
            errored=True,
            error__contains=["Invalid value for quality."],
        )



# -*- coding: utf-8 -*-
"""
Test loop syntax.
"""
from unittest import TestCase

from pyxform.builder import create_survey_from_xls
from pyxform.tests import utils


class LoopTests(TestCase):
    def test_loop(self):
        path = utils.path_to_text_fixture("another_loop.xls")
        survey = create_survey_from_xls(path, "another_loop")
        self.maxDiff = None
        expected_dict = {
            "name": "another_loop",
            "id_string": "another_loop",
            "sms_keyword": "another_loop",
            "default_language": "default",
            "title": "another_loop",
            "type": "survey",
            "children": [
                {
                    "name": "loop_vehicle_types",
                    "type": "group",
                    "children": [
                        {
                            "label": {"English": "Car", "French": "Voiture"},
                            "name": "car",
                            "type": "group",
                            "children": [
                                {
                                    "label": {
                                        "English": "How many do you have?",
                                        "French": "Combien avoir?",
                                    },
                                    "name": "total",
                                    "type": "integer",
                                },
                                {
                                    "bind": {"constraint": ". <= ../total"},
                                    "label": {
                                        "English": "How many are working?",
                                        "French": "Combien marcher?",
                                    },
                                    "name": "working",
                                    "type": "integer",
                                },
                            ],
                        },
                        {
                            "label": {"English": "Motorcycle", "French": "Moto"},
                            "name": "motor_cycle",
                            "type": "group",
                            "children": [
                                {
                                    "label": {
                                        "English": "How many do you have?",
                                        "French": "Combien avoir?",
                                    },
                                    "name": "total",
                                    "type": "integer",
                                },
                                {
                                    "bind": {"constraint": ". <= ../total"},
                                    "label": {
                                        "English": "How many are working?",
                                        "French": "Combien marcher?",
                                    },
                                    "name": "working",
                                    "type": "integer",
                                },
                            ],
                        },
                    ],
                },
                {
                    "children": [
                        {
                            "bind": {"jr:preload": "uid", "readonly": "true()"},
                            "name": "instanceID",
                            "type": "calculate",
                        }
                    ],
                    "control": {"bodyless": True},
                    "name": "meta",
                    "type": "group",
                },
            ],
        }
        self.assertEquals(survey.to_json_dict(), expected_dict)
